<?php
// namespace administrator\components\com_jmap\plugins;
/**
 * @package JMAP::EXTERNALPLUGINS::administrator::components::com_jmap
 * @subpackage plugins
 * @author Joomla! Extensions Store
 * @copyright (C) 2015 - Joomla! Extensions Store
 * @license GNU/GPLv2 http://www.gnu.org/licenses/gpl-2.0.html
 */
defined ( '_JEXEC' ) or die ( 'Restricted access' );

/**
 * External plugin data source
 * It's the concrete implementation of the interface JMapFilePlugin
 * that retrieves data in an arbitrary way or resource and returns them
 * following a specific format to render the sitemap in every format supported HTML, XML, etc
 *
 * @package JMAP::FRAMEWORK::components::com_jmap
 * @subpackage plugins
 * @since 3.3
 */
class JMapFilePluginHikashop implements JMapFilePlugin {
	/**
	 * Retrieves records for the plugin data source using whatever way and resource is required
	 * Formats and returns an associative array of data based on the following scheme  
	 *
	 * @param JRegistry The object holding configuration parameters for the plugin and data source
	 * @param JDatabase $db The database connector object
	 * @param JMapModel $sitemapModel The sitemap model object reference, it's needed to manage limitStart, limitRows properties and affected_rows state
	 *        	
	 * @return array
	 * This function must return an associative array as following:
	 * $returndata['items'] -> It's the mandatory objects array of elements, it must contain at least title and routed link fields
	 * $returndata['items_tree'] -> Needed to render elements grouped by cats with a nested tree, not mandatory
	 * $returndata['categories_tree'] -> Needed to render elements grouped by cats with a nested tree, not mandatory
	 * 
	 * $returndata['items'] must contain records objects with following properties (* = required)
	 * 						->title * A string for the title
	 * 						->link * A string for the link
	 * 						->lastmod (used for XML sitemap) A date string in MySql format yyyy-mm-dd hh:ii:ss
	 * 						->metakey (used for Google news sitemap) A string for metakeys of each record
	 * 						->publish_up (used for Google news sitemap) A date string in MySql format yyyy-mm-dd hh:ii:ss
	 * 						->access (used for Google news sitemap, >1 = registration access) An integer for Joomla! access level of each record
	 * 
	 * $returndata['items_tree'] must be a numerical array that groups items by the containing category id, the index of the array is the category id 
	 * 
	 * $returndata['categories_tree'] must be a numerical array that groups categories by parent category, the index of the array is the category parent id,
	 * 								  the elements of the array must be records objects representing categories with following properties (* = required)
	 * 						->category_id * An integer for the category ID
	 * 						->category_title * A string for the category title
	 * 						->category_link * A string for the category link
	 * 						->lastmod (used for XML sitemap) A date string in MySql format yyyy-mm-dd hh:ii:ss
	 */
	public function getSourceData(JRegistry $pluginParams, JDatabase $db, JMapModel $sitemapModel) {
		// Check if the extension is installed
		if(!file_exists(JPATH_SITE . '/components/com_hikashop')) {
			throw new JMapException(JText::sprintf('COM_JMAP_ERROR_EXTENSION_NOTINSTALLED', 'Hikashop'), 'warning');
		}
		
		// The associative array holding the returned data
		$returndata = array();
		
		// Get default site language
		$hikashopSiteLang = JFactory::getLanguage()->getTag();
		
		// Get plugin params
		$publishupFallback = $pluginParams->get('publish_up_fallback', 1);
		$fallbackItemid = $pluginParams->get('fallback_itemid', null) ? '&Itemid=' . (int)$pluginParams->get('fallback_itemid') : null;
		$sefLanguage = $pluginParams->get('include_language_sef', 0) ? JMapLanguageMultilang::getCurrentSefLanguage() . '/' : null;
		$excludeCatJoinXml = $pluginParams->get('exclude_cat_join_xml', 0);
		
		// Include only main products
		$includeMain = $pluginParams->get('include_main', 1);
		$includeOnlyMainProducts = null;
		if($includeMain) {
			$includeOnlyMainProducts = "\n AND #__hikashop_product.product_type = 'main'";
		}
		
		// Exclude free products
		$excludeFree = $pluginParams->get('exclude_free', 0);
		$excludeFreeProductsJoin = null;
		$excludeFreeProductsAnd = null;
		$distinct = null;
		if($excludeFree) {
			$excludeFreeProductsJoin = "\n JOIN #__hikashop_price ON #__hikashop_product.product_id = #__hikashop_price.price_product_id";
			$excludeFreeProductsAnd = "\n AND #__hikashop_price.price_value > '0'";
			$distinct = " DISTINCT";
		}
		
		$includeCatsFilters = null;
		$includedCats = $pluginParams->get('included_cats', null);
		if($includedCats) {
			$arrayIncludedCats = explode(',', $includedCats);
			JArrayHelper::toInteger($arrayIncludedCats);
			$includeCatsFilters = "\n AND `#__hikashop_category`.`category_id` IN ( " . implode(',', $arrayIncludedCats) . ")";
		}
		
		$excludeCatsFilters = null;
		$excludedCats = $pluginParams->get('excluded_cats', null);
		if($excludedCats) {
			$arrayExcludedCats = explode(',', $excludedCats);
			JArrayHelper::toInteger($arrayExcludedCats);
			$excludeCatsFilters = "\n AND `#__hikashop_category`.`category_id` NOT IN ( " . implode(',', $arrayExcludedCats) . ")";
		}
		
		if($excludeCatJoinXml && JFactory::getApplication()->input->get ( 'format', 'html' ) != 'html') {
			$itemsQuery ="SELECT" . $distinct .
						"\n #__hikashop_product.product_canonical," .
						"\n #__hikashop_product.product_name AS " . $db->quoteName('title') . "," .
						"\n #__hikashop_product.product_alias AS " . $db->quoteName('alias') . "," .
						"\n #__hikashop_product.product_id," .
						"\n #__hikashop_product.product_code," .
						"\n #__hikashop_product.product_modified" .
						"\n FROM #__hikashop_product" .
						$excludeFreeProductsJoin .
						"\n WHERE" .
						"\n #__hikashop_product.product_published = 1" .
						$includeOnlyMainProducts .
						$excludeFreeProductsAnd .
						"\n ORDER BY" .
						"\n #__hikashop_product.product_name ASC";
		} else {
			$itemsQuery ="SELECT" . $distinct .
						"\n #__hikashop_product.product_canonical," .
						"\n #__hikashop_product.product_name AS " . $db->quoteName('title') . "," .
						"\n #__hikashop_product.product_alias AS " . $db->quoteName('alias') . "," .
						"\n #__hikashop_product.product_id," .
						"\n #__hikashop_product.product_code," .
						"\n #__hikashop_product.product_modified," .
						"\n #__hikashop_category.category_name," .
						"\n #__hikashop_category.category_id" .
						"\n FROM #__hikashop_product" .
						"\n JOIN #__hikashop_product_category ON #__hikashop_product.product_id = #__hikashop_product_category.product_id" .
						"\n JOIN #__hikashop_category ON #__hikashop_product_category.category_id = #__hikashop_category.category_id" .
						$excludeFreeProductsJoin .
						"\n WHERE" .
						"\n #__hikashop_category.category_published = 1" .
						"\n AND #__hikashop_category.category_type = 'product'" .
						"\n AND #__hikashop_product.product_published = 1" .
						$includeOnlyMainProducts .
						$excludeFreeProductsAnd .
						$includeCatsFilters .
						$excludeCatsFilters .
						"\n ORDER BY" .
						"\n #__hikashop_category.category_left ASC," .
						"\n #__hikashop_product.product_name ASC";
		}
		
		// Check if a limit for query rows has been set, this means we are in precaching process by JS App client
		if(!$sitemapModel->limitRows) {
			$items = $db->setQuery($itemsQuery)->loadObjectList();
		} else {
			$items = $db->setQuery($itemsQuery, $sitemapModel->limitStart, $sitemapModel->limitRows)->loadObjectList();
		}
		
		if ($db->getErrorNum ()) {
			throw new JMapException(JText::sprintf('COM_JMAP_ERROR_RETRIEVING_DATA_FROM_PLUGIN_DATASOURCE', $db->getErrorMsg()), 'warning');
		}
		
		// Detected a precaching call, we have to store in the model state the number of affected rows for the JS application
		if($sitemapModel->limitRows) {
			$sitemapModel->setState('affected_rows', $db->getAffectedRows());
		}
		
		// Include the extension route helper
		if(file_exists(JPATH_SITE . '/components/com_hikashop/helpers/route.php')) {
			include_once (JPATH_SITE . '/components/com_hikashop/helpers/route.php');
		}
		
		// Route links for each record
		if(count($items)) {
			$itemsByCats = array();
			foreach ($items as $item) {
				if($item->product_canonical) {
					// Ensure that products links are relative
					$item->product_canonical = str_replace(JUri::base(), '', $item->product_canonical);
					$item->link = '/' . $sefLanguage . ltrim(htmlspecialchars($item->product_canonical, null, 'UTF-8', false), '/');
				} else {
					if(version_compare(JVERSION, '3.1.4', '>') && method_exists(hikashopTagRouteHelper, 'buildLanguageLookup')) {
						$item->link = JRoute::_(hikashopTagRouteHelper::getProductRoute($item->product_id . ':' . $item->alias, $item->category_id, $hikashopSiteLang));
					} else {
						$item->link = JRoute::_('index.php?option=com_hikashop&ctrl=product&task=show&cid=' . $item->product_id . '&name=' . $item->product_code . $fallbackItemid);
					}
				}
				
				// Check if a valid last modified date is available for this record, fallback on the publish date
				$item->lastmod = date('Y-m-d H:i:s', $item->product_modified);
				// First pass organize items by containing cats
				@$itemsByCats[$item->category_id][] = $item;
			}
			$returndata['items'] = $items; // Assign items
			$returndata['items_tree'] = $itemsByCats; // Assign items grouped by category
		}
		
		// Build the categories tree
		$catsQuery = "SELECT DISTINCT" . 
					 "\n #__hikashop_category.category_id AS " . $db->quoteName('category_id') . "," .
					 "\n #__hikashop_category.category_alias AS " . $db->quoteName('category_alias') . "," .
					 "\n #__hikashop_category.category_name AS " . $db->quoteName('category_title') . "," .
					 "\n #__hikashop_category.category_canonical" .
					 "\n FROM " . $db->quoteName('#__hikashop_category') .
					 "\n WHERE #__hikashop_category.category_published = 1" .
					 "\n AND #__hikashop_category.category_type = 'product'" .
					 $includeCatsFilters .
					 $excludeCatsFilters;
		$totalItemsCats = $db->setQuery($catsQuery)->loadObjectList();
		if ($db->getErrorNum ()) {
			throw new JMapException(JText::sprintf('COM_JMAP_ERROR_RETRIEVING_DATA_FROM_PLUGIN_DATASOURCE', $db->getErrorMsg()), 'warning');
		}
		 
		$catsTreeQuery = "SELECT" .
						 "\n #__hikashop_category.category_parent_id AS " . $db->quoteName('parent') . "," .
						 "\n #__hikashop_category.category_id AS " . $db->quoteName('child') .
 						 "\n FROM " .$db->quoteName('#__hikashop_category') .
 						 "\n WHERE #__hikashop_category.category_published = 1" .
 						 "\n AND #__hikashop_category.category_type = 'product'" .
 						 $includeCatsFilters .
					 	 $excludeCatsFilters;
		$totalItemsCatsTree = $db->setQuery($catsTreeQuery)->loadAssocList('child');
		if ($db->getErrorNum ()) {
			throw new JMapException(JText::sprintf('COM_JMAP_ERROR_RETRIEVING_DATA_FROM_PLUGIN_DATASOURCE', $db->getErrorMsg()), 'warning');
		}

		// Second pass organize the categories tree by parent -> children
		$catsTree = array();
		if(count($totalItemsCats)) {
			// Check if categories must be linkable
			$linkableCats = $pluginParams->get('linkable_content_cats', 0);
			foreach ($totalItemsCats as &$childCat) {
				if($linkableCats) {
					// Ensure that categories links are relative
					$childCat->category_canonical = str_replace(JUri::base(), '', $childCat->category_canonical);
					$childCat->category_link = '/' . $sefLanguage . ltrim($childCat->category_canonical, '/');
				}
				$parentCat = $totalItemsCatsTree[$childCat->category_id]['parent'];
				$catsTree[$parentCat][] = $childCat;
			}
		}
		
		$array_keys = array_keys($catsTree);
		$firstKey = $array_keys[0];
		$firstCat = array_slice($catsTree, 0, 1, true);
		$catsTree[0] = $firstCat[$firstKey];
		unset($catsTree[$firstKey]);
		$returndata['categories_tree'] = $catsTree; // Assign the categories tree
		
		return $returndata;
	}
}